import os
import time

from PySide2.QtWidgets import QMainWindow, QFileDialog, QMessageBox
from PySide2.QtGui import QDoubleValidator, QIntValidator
from PySide2 import QtCore

from genexpa.uis.uis.MainWindowUI import Ui_MainWindow
from genexpa.uis.WaitPopup import WaitPopup
from genexpa.uis.TableDisplayDialog import TableDisplayDialog

from genexpa.core.actions import Actions
from genexpa.core.system import System
from genexpa.core import helpers, explib


class CoherenceScoreModel(QtCore.QAbstractListModel):
    def __init__(self):
        super().__init__()
        self.coherence_list = [] # gene, value
        self.coherence_df = None

    def data(self, index, role):
        if role == QtCore.Qt.DisplayRole:
            gene, value = self.coherence_list[index.row()]

            gene_str = "{:<14}  {:.2f}".format(gene, value)
            return gene_str

    def rowCount(self, index):
        return len(self.coherence_list)


    def clearModel(self):
        if len(self.coherence_list) == 0:
            return None

        self.beginRemoveRows(QtCore.QModelIndex(), 0, len(self.coherence_list)-1)
        self.coherence_list.clear()
        self.coherence_df = None
        self.endRemoveRows()

    def setCoherence(self, df):
        if self.coherence_df is df:
            return

        self.clearModel()

        self.coherence_df = df
        genes = self.coherence_df.columns.get_level_values(0).unique()
        self.beginInsertRows(QtCore.QModelIndex(), 0, len(genes))

        values_sum = 0.0
        for gene in genes:
            value = df[gene].values.mean()
            self.coherence_list.append([
                gene,
                value
            ])
            values_sum += value

        mean = values_sum/len(genes)
        self.coherence_list.append(["Average", mean])

        self.endInsertRows()

class MainWindowApp(QMainWindow):
    def __init__(self, app):
        super().__init__()

        self.actions = Actions()
        self.system = System(self, app, self.actions)

        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.coherence_score_model = CoherenceScoreModel()
        self.ui.listCoherence.setModel(self.coherence_score_model)
        self.ui.listCoherence.setSelectionRectVisible(False)

        self.remove_validator = QIntValidator(0,100)
        self.ui.lineEditRemoveRep.setValidator(self.remove_validator)
        self.ui.lineEditSampleRepetition.setValidator(self.remove_validator)

        self.ui.lineEditConfidence.setText(str(self.system.parameters_data.confidence))
        self.ui.lineEditRemoveRep.setText(str(self.system.parameters_data.remove_rep))
        self.ui.lineEditSampleRepetition.setText(str(self.system.parameters_data.sample_rep))

        self.ui.checkSelectBest.setChecked(self.system.parameters_data.select_best)

        self.ui.comboBoxNormalizationAlgorithm.setModel(self.system.parameters_data.normalization_algorithms_model)
        self.ui.comboBoxStatModel.setModel(self.system.parameters_data.stat_algorithms_model)

        self.input_data = None

        self.init_connections()

        self.update_gui()

    def init_connections(self):
        self.ui.actionRead_from_combined_input.triggered.connect(self.actions.read_from_combined_input)
        self.ui.actionLoad_Quantified.triggered.connect(self.actions.load_quantified)
        self.ui.actionActivate_license.triggered.connect(self.actions.open_licence_popup)

        self.system.reference_genes_data.changed.connect(self.reference_genes_data_changed)
        self.system.target_genes_data.changed.connect(self.target_genes_data_changed)
        self.system.line_models_data.changed.connect(self.line_models_data_changed)
        self.system.quantitive_data.changed.connect(self.update_gui)
        self.system.results_data.changed.connect(self.update_gui)

        self.ui.buttonShowRef.clicked.connect(self.show_ref_table)
        self.ui.buttonShowTarget.clicked.connect(self.show_target_table)
        self.ui.buttonShowModels.clicked.connect(self.show_model_list)
        self.ui.buttonShowQuantitive.clicked.connect(self.show_quantitivie)

        self.ui.buttonRemoveQuantitive.clicked.connect(self.system.quantitive_data.clear)

        self.ui.buttonRunCalculations.clicked.connect(self.run_calculations)

        self.ui.lineEditConfidence.textChanged.connect(
            lambda x: self.parameters_changed(x, "confidence"))

        self.ui.lineEditRemoveRep.textChanged.connect(
            lambda x:self.parameters_changed(x, "remove_rep"))

        self.ui.lineEditSampleRepetition.textChanged.connect(
            lambda x: self.parameters_changed(x, "sample_rep"))

        self.ui.checkSelectBest.stateChanged.connect(
            lambda x: self.parameters_changed(x, "select_best"))

        self.ui.buttonShowBestRef.clicked.connect(self.show_best_ref)
        self.ui.buttonShowRQ.clicked.connect(self.show_RQ)
        self.ui.buttonShowStatistic.clicked.connect(self.show_statistic)
        #self.ui.buttonShowCoherence.clicked.connect(self.show_coherence_score)

        self.ui.buttonExportResults.clicked.connect(self.export_results)
        self.ui.buttonExportGraphs.clicked.connect(self.export_graphs)

    def parameters_changed(self, value, parameter_str):
        self.system.parameters_data.update(value, parameter_str)
        self.update_gui()

    def reference_genes_data_changed(self):
        self.ui.labelRefCnt.setText(str(self.system.reference_genes_data.get_genes_count()))
        self.update_gui()

    def target_genes_data_changed(self):
        self.ui.labelTargetCnt.setText(str(self.system.target_genes_data.get_genes_count()))
        self.update_gui()

    def line_models_data_changed(self):
        self.ui.labelModelsCnt.setText(str(self.system.line_models_data.get_models_cout()))
        self.update_gui()


    def show_ref_table(self):
        self.ref_table = TableDisplayDialog(self.system.mainWindowApp,
                                            self.system.reference_genes_data.df,
                                            "Reference genes data")
        self.ref_table.show()

    def show_target_table(self):
        self.target_table = TableDisplayDialog(self.system.mainWindowApp,
                                               self.system.target_genes_data.df,
                                               "Target genes data")
        self.target_table.show()

    def show_model_list(self):
        self.models_table =TableDisplayDialog(self.system.mainWindowApp,
                                              self.system.line_models_data.as_df(),
                                              "Lines models data",
                                              [0])
        self.models_table.show()

    def show_quantitivie(self):
        self.quantitive_table = TableDisplayDialog(self.system.mainWindowApp,
                                                   self.system.quantitive_data.df,
                                                   "Quantitive gene data")
        self.quantitive_table.show()

    def show_best_ref(self):
        self.best_table = TableDisplayDialog(self.system.mainWindowApp,
            self.system.results_data.df_best_ref,
            "Best reference gene/pair results data",
            [0]
        )
        self.best_table.show()

    def show_RQ(self):
        self.RQ_table = TableDisplayDialog(self.system.mainWindowApp,
            self.system.results_data.df_RQ,
            "RQ results data"
        )
        self.RQ_table.show()

    def show_statistic(self):
        self.stat_table = TableDisplayDialog(self.system.mainWindowApp,
            self.system.results_data.df_stat,
            "Statistic results data"
        )
        self.stat_table.show()

    def show_coherence_score(self):
        self.coherence_table = TableDisplayDialog(self.system.mainWindowApp,
            self.system.results_data.df_coherence.T,
            "Coherence score results data",
            str_col = list(range(len(self.system.results_data.df_coherence.columns)))
        )
        self.coherence_table.show()

    def run_calculations(self):
        if not (self.system.input_valid() and self.system.parameters_data.is_valid()):
            return None

        df_ref = explib.append_group_index(self.system.reference_genes_data.df)

        df_target = self.system.target_genes_data.df
        df_ref_norm = self.system.quantitive_data.df

        models = self.system.line_models_data.models

        remove_rep = self.system.parameters_data.remove_rep
        sample_rep = self.system.parameters_data.sample_rep

        confidence = self.system.parameters_data.confidence

        stat_mode = self.ui.comboBoxStatModel.currentIndex()

        ind = self.ui.comboBoxNormalizationAlgorithm.currentIndex()
        norm_alog = self.system.parameters_data.normalization_algorithms[ind]

        apply_remove_mask = self.ui.checkSelectBest.isChecked()
        wp = WaitPopup(self.system)

        try:
            wp.start()
            res = explib.full_analyze(df_ref, df_target, models, remove_rep, sample_rep, stat_mode, confidence, df_ref_norm, norm_alog)

        except:
            import traceback
            print(traceback.format_exc())
            res = None
            helpers.popup("Run ERROR", "Error occured during analyze step,\npleas double check you input data", "critical")

        wp.calculation_finished()

        if res is not None:
            self.system.results_data.set_results(res, apply_remove_mask)

        else:
            self.system.results_data.clear()

    def export_results(self):
        export_dfs = self.system.results_data.get_all_df()
        if export_dfs is None:
            return None

        filename, filter = QFileDialog.getSaveFileName(
            parent=self, caption='Select result file', dir='.',
            filter='Microsoft Excel 2007-2013 XML (*.xlsx)')

        if filename == "":
            return None

        if filename.split(".")[-1] != "xlsx":
            filename = filename + ".xlsx"
        explib.full_excel_export(filename, export_dfs)

    def export_graphs(self):
        export_dfs = self.system.results_data.get_all_df()
        if export_dfs is None:
            return

        dir_name = QFileDialog.getExistingDirectory(
            parent=self.system.mainWindowApp,
            caption='Select directory to save generated graphs',
            dir='.')

        if dir_name == "":
            return None

        RQ_df = export_dfs[2]
        stat_df = export_dfs[3]

        wp = WaitPopup(self.system)
        wp.start()
        explib.create_boxplots(RQ_df, stat_df, dir_name)
        try:
            pass
        except Exception as e:
            helpers.popup("Graph export ERROR", "Error occured during graph export", 'critical')
            print(e)

        wp.calculation_finished()


    def update_gui(self):
        input_valid = self.system.input_valid()
        if input_valid:
            self.ui.buttonShowRef.setEnabled(True)
            self.ui.buttonShowTarget.setEnabled(True)
            self.ui.buttonShowModels.setEnabled(True)
        else:
            self.ui.buttonShowRef.setEnabled(False)
            self.ui.buttonShowTarget.setEnabled(False)
            self.ui.buttonShowModels.setEnabled(False)
            self.ui.buttonShowQuantitive.setEnabled(False)
            self.ui.labelQantitive.setText("Not loaded")

        quanta_valid = self.system.quantitive_data.is_valid()
        if quanta_valid:
            self.ui.buttonShowQuantitive.setEnabled(True)
            self.ui.buttonRemoveQuantitive.setEnabled(True)
            self.ui.labelQantitive.setText("Loaded!")
        else:
            self.ui.buttonRemoveQuantitive.setEnabled(False)

        parameters_valid = self.system.parameters_data.is_valid()

        if (input_valid and parameters_valid):
            self.ui.buttonRunCalculations.setEnabled(True)
        else:
            self.ui.buttonRunCalculations.setEnabled(False)

        results_valid = self.system.results_data.is_valid()

        if results_valid:
            self.ui.groupBoxResultsSummary.setEnabled(True)
            #self.ui.labelCoherenceScore.setText("{0:.3f}".format(self.system.results_data.get_summary_coherence()))
            self.coherence_score_model.setCoherence(self.system.results_data.df_coherence)

        else:
            self.ui.groupBoxResultsSummary.setEnabled(False)



from PySide2.QtWidgets import QDialog

from genexpa.uis.uis.WaitPopupUI import Ui_Form


class WaitPopup(QDialog):
    def __init__(self, system):
        self.system = system
        super(WaitPopup, self).__init__(system.mainWindowApp)

        self.ui = Ui_Form()
        self.ui.setupUi(self)

        self.close_allowed = False

    def start(self):
        self.open()
        self.system.app.processEvents()

    def closeEvent(self, event):
        if self.close_allowed:
            super(WaitPopup, self).closeEvent(event)

        else:
            event.ignore()

    def calculation_finished(self):
        self.close_allowed = True
        self.close()



import pandas as pd
import numpy as np

from PySide2.QtCore import QObject, Signal
from PySide2.QtWidgets import QFileDialog, QMessageBox

from PySide2.QtGui import QStandardItemModel, QStandardItem

from genexpa.uis.ReadFromCombinedDialog import ReadFromCombinedDialog
from genexpa.core import explib, helpers

class GenesData(QObject):
    changed = Signal()

    def __init__(self, parent, actions):
        super().__init__(parent)
        self.actions = actions

        self.df = None

    def update(self, df):
        self.df = df
        self.changed.emit()

    def get_genes_count(self):
        if self.df is None:
            return 0
        return self.df.shape[0]

    def get_lines_cout(self):
        if self.df is None:
            return 0
        return len(self.df.columns.unique())

    def is_valid(self):
        return self.get_genes_count() > 0 and self.get_lines_cout() > 0

class LineModelsData(QObject):
    changed = Signal()

    def __init__(self, parent, actions):
        super().__init__(parent)
        self.actions = actions
        self.models = None

    def update(self, models):
        self.models = models
        self.changed.emit()

    def get_models_cout(self):
        if self.models is None:
            return 0
        return len(self.models)

    def as_df(self):
        if self.models is None:
            models = []
        else:
            models = [" ".join(m) for m in self.models]
            return pd.DataFrame(models, columns=["Cell lines"])

    def is_valid(self):
        return self.get_models_cout() > 0

class QuantitiveData(QObject):
    changed = Signal()

    def __init__(self, parent, actions):
        super().__init__(parent)
        self.actions = actions
        self.df = None

    def update(self, df):
        self.df = df
        self.changed.emit()

    def load_file(self, path, df_inputdata):
        df = explib.read_combined_input(path)

        if len(df.columns) != len(df_inputdata.columns):
            base_data = " ".join(df_inputdata.columns)
            quanta_data = " ".join(df.columns)
            error_str = "Not matching in cell lines names\n\n"
            error_str += "Input data (samples = {}):\n{}\n\n".format(len(df_inputdata.columns), base_data)
            error_str += "Quantified data (samples = {}):\n{}""".format(len(df.columns), quanta_data)

            helpers.popup("Bad input/quantified data", error_str)
            return None

        if not all(df.columns == df_inputdata.columns):
            missmatch = np.logical_not(df.columns == df_inputdata.columns)
            base_data = " ".join(df_inputdata.columns[missmatch])
            quanta_data = " ".join(df.columns[missmatch])
            error_str = """Miss match in cell lines names\nmiss matched normal data:\n{}\n\nmiss matched quantified data:\n{}""".format(base_data, quanta_data)

            helpers.popup("Bad input/quantified data", error_str)
            return None

        missing_genes = []

        for ref_gene in df_inputdata.index:
            if ref_gene not in df.index:
                missing_genes.append(ref_gene)

        if len(missing_genes) != 0:
            error_str = "Not all reference genes are present in quantified data\n\n"
            error_str += "Genes in input data\n{}\n\n".format(" ".join(df_inputdata.index))
            error_str += "Genes in quantified data:\n{}\n\n".format(" ".join(df.index))
            error_str += "Missing genes:\n{}""".format(" ".join(missing_genes))

            helpers.popup("Bad input data", error_str)
            return None

        self.df = df
        self.changed.emit()

    def clear(self):
        self.df = None
        self.changed.emit()

    def is_valid(self):
        if self.df is None:
            return False
        else:
            return True

class ResultsData(QObject):
    changed = Signal()
    def __init__(self, parent, actions):
        super().__init__(parent)
        self.actions = actions

        self.df_mean = None
        self.df_best_ref = None
        self.df_RQ = None
        self.df_stat = None
        self.df_comparison = None

        self.df_coherence = None

    def set_results(self, res, apply_remove_mask):
        self.df_mean, self.df_best_ref, self.df_RQ, self.df_stat, self.df_comparison = res

        if apply_remove_mask:
            mask = explib.get_best_series_remove(self.df_best_ref)
            mask_exp = explib.expand_mask(mask, self.df_RQ)
            self.df_mean = self.df_mean.loc[mask]
            self.df_best_ref = self.df_best_ref.loc[mask]
            self.df_RQ = self.df_RQ.loc[mask_exp]
            self.df_comparison = self.df_comparison[mask]

            _df_stat = self.df_stat.loc[mask]
            if hasattr(self.df_stat, "alpha"):
                _df_stat.alpha = self.df_stat.alpha
            if hasattr(self.df_stat, "stat_model"):
                _df_stat.stat_model = self.df_stat.stat_model

            self.df_stat = _df_stat

        self.df_coherence = explib.calc_coherence(self.df_comparison)

        self.changed.emit()

    def clear(self):
        self.df_mean = None
        self.df_best_ref = None
        self.df_RQ = None
        self.df_stat = None
        self.df_comparison = None

        self.df_coherence = None

        self.changed.emit()

    def get_summary_coherence(self):
        if self.df_coherence is not None:
            return self.df_coherence.mean(axis=1).values[0]
        else:
            return 0.0

    def is_valid(self):
        if self.df_mean is None:
            return False
        return True

    def get_all_df(self):
        return [self.df_mean, self.df_best_ref, self.df_RQ, self.df_stat, self.df_comparison, self.df_coherence]

class ParametersData(QObject):
    changed = Signal()
    def __init__(self, parent, actions):
        super().__init__(parent)
        self.actions = actions

        self.normalization_algorithms = ["Normfinder"] #["Normfinder", "GeNorm"]
        self.normalization_algorithms_model = QStandardItemModel (len(self.normalization_algorithms), 1)
        for i, norm_algo in enumerate(self.normalization_algorithms):
            item = QStandardItem(norm_algo)
            self.normalization_algorithms_model.setItem(i, 0, item)

        self.stat_algorithms = ["Pairwise t-test, Holm adjustment",
                                "Kolmogorov-Smirnov/Kruskal-Wallis, post: Dunn’s test",
                                "Mann-Whitney/Kruskal-Wallis, post: Dunn’s test",
                                "Kruskal-Wallis"]
        #self.stat_algorithms_modes = ["kw", "ks", "welch"]
        self.stat_algorithms_model = QStandardItemModel(len(self.stat_algorithms), 1)
        for i, stat_algo in enumerate(self.stat_algorithms):
            item = QStandardItem(stat_algo)
            self.stat_algorithms_model.setItem(i, 0, item)

        # internal data
        self.confidence = 0.05
        self.remove_rep = 0
        self.sample_rep = 3
        self.select_best = False

    def is_valid(self):
        if self.confidence < 0.0:
            return False

        if self.remove_rep < 0:
            return False

        if self.sample_rep < 0:
            return False

        return True

    def update(self, value, param_str):
        #self.sel_normalization_algorithm =
        if param_str == "confidence":
            try:
                self.confidence = float(value)
            except:
                self.confidence = -1.0

        elif param_str == "remove_rep":
            try:
                self.remove_rep = int(value)
            except:
                self.remove_rep = -1

        elif param_str == "sample_rep":
            try:
                self.sample_rep = int(value)
            except:
                self.sample_rep = -1

        elif param_str == "select_best":
            self.select_best = bool(value)

        self.changed.emit()


class System(QObject):
    def __init__(self, parent, app, actions):
        super().__init__(parent)
        self.mainWindowApp = self.parent()

        self.app = app

        self.actions = actions

        self.reference_genes_data = GenesData(self, self.actions)
        self.target_genes_data = GenesData(self, self.actions)
        self.line_models_data = LineModelsData(self, self.actions)
        self.quantitive_data = QuantitiveData(self, self.actions)
        self.results_data = ResultsData(self, self.actions)

        self.parameters_data = ParametersData(self, self.actions)

        self.read_from_combined_dialog = ReadFromCombinedDialog(self.mainWindowApp,
                                                                self.actions,
                                                                self)

        self.init_connections()

    def init_connections(self):
        self.actions.read_from_combined_input.connect(self.read_from_combined_input)
        self.actions.load_quantified.connect(self.load_quantified)

    def read_from_combined_input(self):
        accepted = self.read_from_combined_dialog.exec_()

        if accepted == 0:
            return None

        combined_data = self.read_from_combined_dialog.get_dataframes()

        if combined_data is None:
            return None

        df_ref, df_target, lines_models = combined_data

        self.reference_genes_data.update(df_ref)
        self.target_genes_data.update(df_target)
        self.line_models_data.update(lines_models)

    def load_quantified(self):
        if not self.input_valid():
            helpers.popup("Input data not loaded!",
                          "Before loading quantified data, first load proper input data!",
                          "warning")
            return None

        filename, filter = QFileDialog.getOpenFileName(
            parent=self.mainWindowApp,
            caption='Open quantified input file',
            dir='.',
            filter='*.xlsx *.xls')

        if filename == "":
            return None

        succes = self.quantitive_data.load_file(filename, self.reference_genes_data.df)
        if succes:
            self.mainWindowApp.update_gui()

    def input_valid(self):
        return (self.reference_genes_data.is_valid() and
        self.target_genes_data.is_valid() and
        self.line_models_data.is_valid())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'LineModelDialog.ui',
# licensing of 'LineModelDialog.ui' applies.
#
# Created: Sat Jan  4 23:05:16 2020
#      by: pyside2-uic  running on PySide2 5.12.0
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(375, 311)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.listLines = QtWidgets.QListView(Dialog)
        self.listLines.setObjectName("listLines")
        self.horizontalLayout.addWidget(self.listLines)
        self.buttonAddLine = QtWidgets.QPushButton(Dialog)
        self.buttonAddLine.setObjectName("buttonAddLine")
        self.horizontalLayout.addWidget(self.buttonAddLine)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_2 = QtWidgets.QLabel(Dialog)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.labelModelText = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setWeight(75)
        font.setBold(True)
        self.labelModelText.setFont(font)
        self.labelModelText.setObjectName("labelModelText")
        self.horizontalLayout_2.addWidget(self.labelModelText)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.buttonDiscard = QtWidgets.QPushButton(Dialog)
        self.buttonDiscard.setObjectName("buttonDiscard")
        self.horizontalLayout_3.addWidget(self.buttonDiscard)
        self.buttonAccept = QtWidgets.QPushButton(Dialog)
        self.buttonAccept.setObjectName("buttonAccept")
        self.horizontalLayout_3.addWidget(self.buttonAccept)
        self.verticalLayout.addLayout(self.horizontalLayout_3)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtWidgets.QApplication.translate("Dialog", "Dialog", None, -1))
        self.buttonAddLine.setText(QtWidgets.QApplication.translate("Dialog", "Add line to model", None, -1))
        self.label_2.setText(QtWidgets.QApplication.translate("Dialog", "Selected model:", None, -1))
        self.labelModelText.setText(QtWidgets.QApplication.translate("Dialog", "-", None, -1))
        self.buttonDiscard.setText(QtWidgets.QApplication.translate("Dialog", "Discard", None, -1))
        self.buttonAccept.setText(QtWidgets.QApplication.translate("Dialog", "Accept", None, -1))




import numpy as np
import pandas as pd

def normfinder(dat0, textfile=None, ctVal=False, pStabLim=0.25):
    if textfile:
        dat0 = pd.read_csv(textfile, index_col=0, sep=r"\s+")
    
    # ----------------------
    ntotal = dat0.shape[1]
    k0 = dat0.shape[0]

    ngenes = k0-1
    genenames = dat0.index[:-1]
    grId = dat0.values[-1].astype(int)

    dat = dat0.values[:-1]

    if not ctVal:
        dat = np.log2(dat)
    uq_grId = np.unique(grId)
    ngr = len(uq_grId)

    nsamples = []
    for _id in uq_grId:
        _shape = dat[:,grId == _id].shape
        nsamples.append(_shape[1])
    nsamples = np.array(nsamples)

    # MakeStab(dat)
    da = dat

    sampleavg = np.mean(dat, axis=0)

    genegroupavg = []
    groupavg = []

    # Group averages
    for gr_id in uq_grId:
        gr_ave = np.mean(dat[:, grId == gr_id], axis=1)
        genegroupavg.append(gr_ave)
        groupavg.append(np.mean(gr_ave))

    genegroupavg = np.array(genegroupavg).T
    groupavg = np.array(groupavg)


    # Variances
    GGvar = np.zeros((ngenes, ngr))
    for group_index, gr_name in enumerate(uq_grId):
        grset = grId == gr_name
        a = np.zeros(ngenes)

        for gene_index in range(ngenes):
            a[gene_index] = np.sum(
                (da[gene_index, grset]
                - genegroupavg[gene_index, group_index]
                - sampleavg[grset]
                + groupavg[group_index])**2)/ (nsamples[group_index]-1)

        GGvar[:,group_index] = (a-np.sum(a)/(ngenes*ngenes-ngenes))/(1-2/ngenes)


    # Change possible negative values
    genegroupMinvar = np.zeros((ngenes, ngr))
    for group_index, gr_name in enumerate(uq_grId):
        grset = grId == gr_name
        z = da[:,grset]

        for gene_index in range(ngenes):
            varpair = np.zeros(ngenes, dtype=float)
            for gene1_index in range(ngenes):
                varpair[gene1_index] = np.var(z[gene_index,:]-z[gene1_index,:], ddof=1)

            genegroupMinvar[gene_index, group_index] = np.min(varpair[np.arange(ngenes) != gene_index])/4.0


    # Final variances
    GGvar[GGvar < 0] = genegroupMinvar[GGvar < 0]

    # Old stability measure for each gene is calculated:
    dif = genegroupavg
    difgeneavg = np.mean(dif, axis=1)
    difgroupavg = np.mean(dif, axis=0)
    difavg = np.mean(dif)

    for gene_index in range(ngenes):
        for group_index in range(ngr):
            dif[gene_index, group_index] = (dif[gene_index, group_index]
                                            -difgeneavg[gene_index]
                                            -difgroupavg[group_index]
                                            +difavg)

    nsampMatrix = np.tile(nsamples, (ngenes, 1))
    vardif = GGvar/nsampMatrix

    gamma = np.sum(dif*dif)/((ngr-1)*(ngenes-1))-np.sum(vardif)/(ngenes*ngr)
    gamma = max(gamma, 0)

    difnew = dif*gamma/(gamma+vardif)
    varnew = vardif+gamma*vardif/(gamma+vardif)
    Ostab0 = np.abs(difnew)+np.sqrt(varnew)
    Ostab = np.mean(Ostab0, axis=1)

    # Measure of group differences:
    mud = np.zeros(ngenes)
    for gene_index in range(ngenes):
        mud[gene_index] = 2.0*np.max(np.abs(dif[gene_index,:]))

    # Common variance:
    genevar = np.zeros(ngenes)
    for gene_index in range(ngenes):
        genevar[gene_index] = np.sum((nsamples-1)*GGvar[gene_index,:])/(np.sum(nsamples)-ngr)


    Gsd = np.sqrt(genevar)
    results = [mud, Gsd, Ostab, np.ones(ngenes)*gamma, GGvar, dif]
    results = np.column_stack(results)


    # END MAKE STAB DEFINITION

    # MakeComb2

    def MakeComb2(g1, g2, res):
        gam = res[0,3]

        d1 = res[g1, (4+ngr):(4+ngr+ngr)]
        d2 = res[g2, (4+ngr):(4+ngr+ngr)]
        s1 = res[g1, 4:(4+ngr)]
        s2 = res[g2, 4:(4+ngr)]

        try:
            rho = np.abs(gam*d1/(gam+s1/nsamples)+
                    gam*d2/(gam+s2/nsamples))*np.sqrt(ngenes/(ngenes-2) )/2.0
        except ZeroDivisionError:
            print("TTEST")
            print("(gam+s1/nsamples):{}".format(gam+s1/nsamples))
            print("(gam+s2/nsamples):{}".format(gam+s2/nsamples))
            print("(ngenes-2):{}".format(ngenes-2))
            raise ValueError("val")
        rho = rho + np.sqrt(s1/nsamples+gam*s1/(nsamples*gam+s1) +
                        s2/nsamples+gam*s2/(nsamples*gam+s2))/2

        return np.mean(rho)

    # Main part
    res = results

    #gcand = np.arange(ngenes)[res[:,2]<pStabLim]
    gcand = np.arange(ngenes)[res[:,2] > -np.inf]
    ncand = len(gcand)

    if ncand < 4 :
        if ngenes > 3:
            li = sorted(res[:,2])[3]
            gcand = np.arange(ngenes)[res[:,2] <= li]
            ncand = len(gcand)
        else:
            gcand = np.arange(ngenes)
            ncand = len(gcand)

    vv2 = []
    for g1 in range(ncand-1):
        for g2 in range(g1+1, ncand):
            qmeas = MakeComb2(gcand[g1], gcand[g2], res)

            vv2.append([gcand[g1], gcand[g2], qmeas])

    # return preparation
    vv2_sorted = sorted(vv2, key = lambda x: x[2])
    vv2_named = [[genenames[_[0]], genenames[_[1]], _[2]] for _ in vv2_sorted]

    Ostab_named = [[genenames[_], Ostab[_]] for _ in np.argsort(Ostab)]

    return Ostab_named, vv2_named



from PySide2.QtCore import QObject, Signal

class Actions(QObject):
    read_from_combined_input = Signal()
    load_quantified = Signal()
    open_licence_popup = Signal()

    lock_application = Signal()
    unlock_application = Signal()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'WaitPopup.ui',
# licensing of 'WaitPopup.ui' applies.
#
# Created: Wed Jan 15 11:10:34 2020
#      by: pyside2-uic  running on PySide2 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(341, 172)
        self.horizontalLayout = QtWidgets.QHBoxLayout(Form)
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.label = QtWidgets.QLabel(Form)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle(QtWidgets.QApplication.translate("Form", "Form", None, -1))
        self.label.setText(QtWidgets.QApplication.translate("Form", "Processing data ...\n"
"    Please wait", None, -1))




import sys

from genexpa.uis.MainWindowApp import MainWindowApp
from PySide2.QtWidgets import QApplication


def main():
    app = QApplication(sys.argv)

    main_window = MainWindowApp(app)
    main_window.show()

    sys.exit(app.exec_())

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'MainWindow.ui',
# licensing of 'MainWindow.ui' applies.
#
# Created: Fri Dec  4 11:26:59 2020
#      by: pyside2-uic  running on PySide2 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(794, 879)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.scrollArea = QtWidgets.QScrollArea(self.centralwidget)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 774, 810))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBox = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.buttonShowQuantitive = QtWidgets.QPushButton(self.groupBox)
        self.buttonShowQuantitive.setObjectName("buttonShowQuantitive")
        self.gridLayout.addWidget(self.buttonShowQuantitive, 5, 3, 1, 1)
        self.buttonShowRef = QtWidgets.QPushButton(self.groupBox)
        self.buttonShowRef.setToolTip("")
        self.buttonShowRef.setObjectName("buttonShowRef")
        self.gridLayout.addWidget(self.buttonShowRef, 0, 3, 1, 1)
        self.labelModelsCnt = QtWidgets.QLabel(self.groupBox)
        self.labelModelsCnt.setObjectName("labelModelsCnt")
        self.gridLayout.addWidget(self.labelModelsCnt, 3, 1, 1, 1)
        self.labelTargetCnt = QtWidgets.QLabel(self.groupBox)
        self.labelTargetCnt.setObjectName("labelTargetCnt")
        self.gridLayout.addWidget(self.labelTargetCnt, 1, 1, 1, 1)
        self.buttonShowTarget = QtWidgets.QPushButton(self.groupBox)
        self.buttonShowTarget.setObjectName("buttonShowTarget")
        self.gridLayout.addWidget(self.buttonShowTarget, 1, 3, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.labelRefCnt = QtWidgets.QLabel(self.groupBox)
        self.labelRefCnt.setObjectName("labelRefCnt")
        self.gridLayout.addWidget(self.labelRefCnt, 0, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.groupBox)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 3, 0, 1, 1)
        self.buttonShowModels = QtWidgets.QPushButton(self.groupBox)
        self.buttonShowModels.setObjectName("buttonShowModels")
        self.gridLayout.addWidget(self.buttonShowModels, 3, 3, 1, 1)
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setMouseTracking(False)
        self.label.setWhatsThis("")
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.groupBox)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 5, 0, 1, 1)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.labelQantitive = QtWidgets.QLabel(self.groupBox)
        self.labelQantitive.setObjectName("labelQantitive")
        self.horizontalLayout_6.addWidget(self.labelQantitive)
        self.buttonRemoveQuantitive = QtWidgets.QPushButton(self.groupBox)
        self.buttonRemoveQuantitive.setObjectName("buttonRemoveQuantitive")
        self.horizontalLayout_6.addWidget(self.buttonRemoveQuantitive)
        spacerItem = QtWidgets.QSpacerItem(30, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem)
        self.gridLayout.addLayout(self.horizontalLayout_6, 5, 1, 1, 1)
        self.verticalLayout_3.addWidget(self.groupBox)
        self.groupBox_2 = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_8 = QtWidgets.QLabel(self.groupBox_2)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_4.addWidget(self.label_8)
        self.comboBoxNormalizationAlgorithm = QtWidgets.QComboBox(self.groupBox_2)
        self.comboBoxNormalizationAlgorithm.setMinimumSize(QtCore.QSize(150, 0))
        self.comboBoxNormalizationAlgorithm.setObjectName("comboBoxNormalizationAlgorithm")
        self.comboBoxNormalizationAlgorithm.addItem("")
        self.horizontalLayout_4.addWidget(self.comboBoxNormalizationAlgorithm)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_5 = QtWidgets.QLabel(self.groupBox_2)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout.addWidget(self.label_5)
        self.comboBoxStatModel = QtWidgets.QComboBox(self.groupBox_2)
        self.comboBoxStatModel.setMinimumSize(QtCore.QSize(150, 0))
        self.comboBoxStatModel.setObjectName("comboBoxStatModel")
        self.comboBoxStatModel.addItem("")
        self.horizontalLayout.addWidget(self.comboBoxStatModel)
        self.label_6 = QtWidgets.QLabel(self.groupBox_2)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout.addWidget(self.label_6)
        self.lineEditConfidence = QtWidgets.QLineEdit(self.groupBox_2)
        self.lineEditConfidence.setMaximumSize(QtCore.QSize(50, 16777215))
        self.lineEditConfidence.setObjectName("lineEditConfidence")
        self.horizontalLayout.addWidget(self.lineEditConfidence)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_7 = QtWidgets.QLabel(self.groupBox_2)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_2.addWidget(self.label_7)
        self.lineEditRemoveRep = QtWidgets.QLineEdit(self.groupBox_2)
        self.lineEditRemoveRep.setMaximumSize(QtCore.QSize(40, 16777215))
        self.lineEditRemoveRep.setObjectName("lineEditRemoveRep")
        self.horizontalLayout_2.addWidget(self.lineEditRemoveRep)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem2)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_9 = QtWidgets.QLabel(self.groupBox_2)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_7.addWidget(self.label_9)
        self.lineEditSampleRepetition = QtWidgets.QLineEdit(self.groupBox_2)
        self.lineEditSampleRepetition.setMaximumSize(QtCore.QSize(40, 16777215))
        self.lineEditSampleRepetition.setObjectName("lineEditSampleRepetition")
        self.horizontalLayout_7.addWidget(self.lineEditSampleRepetition)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.checkSelectBest = QtWidgets.QCheckBox(self.groupBox_2)
        self.checkSelectBest.setEnabled(True)
        self.checkSelectBest.setObjectName("checkSelectBest")
        self.horizontalLayout_3.addWidget(self.checkSelectBest)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.verticalLayout_3.addWidget(self.groupBox_2)
        self.widget = QtWidgets.QWidget(self.scrollAreaWidgetContents)
        self.widget.setMaximumSize(QtCore.QSize(16777215, 60))
        self.widget.setObjectName("widget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.widget)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem4 = QtWidgets.QSpacerItem(133, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem4, 1, 2, 1, 1)
        self.buttonRunCalculations = QtWidgets.QPushButton(self.widget)
        self.buttonRunCalculations.setMinimumSize(QtCore.QSize(200, 40))
        self.buttonRunCalculations.setObjectName("buttonRunCalculations")
        self.gridLayout_2.addWidget(self.buttonRunCalculations, 1, 1, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(134, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem5, 1, 0, 1, 1)
        spacerItem6 = QtWidgets.QSpacerItem(20, 110, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem6, 0, 1, 1, 1)
        spacerItem7 = QtWidgets.QSpacerItem(20, 12, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem7, 2, 1, 1, 1)
        self.verticalLayout_3.addWidget(self.widget)
        self.groupBoxResultsSummary = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxResultsSummary.setEnabled(True)
        self.groupBoxResultsSummary.setObjectName("groupBoxResultsSummary")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBoxResultsSummary)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_17 = QtWidgets.QLabel(self.groupBoxResultsSummary)
        self.label_17.setObjectName("label_17")
        self.gridLayout_3.addWidget(self.label_17, 0, 1, 1, 1)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem8)
        self.buttonExportResults = QtWidgets.QPushButton(self.groupBoxResultsSummary)
        self.buttonExportResults.setMinimumSize(QtCore.QSize(200, 40))
        self.buttonExportResults.setObjectName("buttonExportResults")
        self.horizontalLayout_5.addWidget(self.buttonExportResults)
        spacerItem9 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem9)
        self.buttonExportGraphs = QtWidgets.QPushButton(self.groupBoxResultsSummary)
        self.buttonExportGraphs.setEnabled(True)
        self.buttonExportGraphs.setMinimumSize(QtCore.QSize(200, 40))
        self.buttonExportGraphs.setObjectName("buttonExportGraphs")
        self.horizontalLayout_5.addWidget(self.buttonExportGraphs)
        spacerItem10 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem10)
        self.gridLayout_3.addLayout(self.horizontalLayout_5, 9, 1, 1, 2)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        spacerItem11 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem11)
        self.buttonShowBestRef = QtWidgets.QPushButton(self.groupBoxResultsSummary)
        self.buttonShowBestRef.setObjectName("buttonShowBestRef")
        self.verticalLayout_4.addWidget(self.buttonShowBestRef)
        self.buttonShowRQ = QtWidgets.QPushButton(self.groupBoxResultsSummary)
        self.buttonShowRQ.setObjectName("buttonShowRQ")
        self.verticalLayout_4.addWidget(self.buttonShowRQ)
        self.buttonShowStatistic = QtWidgets.QPushButton(self.groupBoxResultsSummary)
        self.buttonShowStatistic.setObjectName("buttonShowStatistic")
        self.verticalLayout_4.addWidget(self.buttonShowStatistic)
        self.gridLayout_3.addLayout(self.verticalLayout_4, 4, 2, 5, 1)
        self.listCoherence = QtWidgets.QListView(self.groupBoxResultsSummary)
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setWeight(50)
        font.setBold(False)
        self.listCoherence.setFont(font)
        self.listCoherence.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.listCoherence.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.listCoherence.setObjectName("listCoherence")
        self.gridLayout_3.addWidget(self.listCoherence, 4, 1, 5, 1)
        self.verticalLayout_3.addWidget(self.groupBoxResultsSummary)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout_4.addWidget(self.scrollArea, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 794, 27))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionRead_from_combined_input = QtWidgets.QAction(MainWindow)
        self.actionRead_from_combined_input.setObjectName("actionRead_from_combined_input")
        self.actionLoad_Quantified = QtWidgets.QAction(MainWindow)
        self.actionLoad_Quantified.setObjectName("actionLoad_Quantified")
        self.actionActivate_license = QtWidgets.QAction(MainWindow)
        self.actionActivate_license.setObjectName("actionActivate_license")
        self.menuFile.addAction(self.actionRead_from_combined_input)
        self.menuFile.addAction(self.actionLoad_Quantified)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionActivate_license)
        self.menubar.addAction(self.menuFile.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtWidgets.QApplication.translate("MainWindow", "MainWindow", None, -1))
        self.groupBox.setTitle(QtWidgets.QApplication.translate("MainWindow", "Input data", None, -1))
        self.buttonShowQuantitive.setText(QtWidgets.QApplication.translate("MainWindow", "Show Table", None, -1))
        self.buttonShowRef.setText(QtWidgets.QApplication.translate("MainWindow", "Show Table", None, -1))
        self.labelModelsCnt.setToolTip(QtWidgets.QApplication.translate("MainWindow", "Cell models count", None, -1))
        self.labelModelsCnt.setStatusTip(QtWidgets.QApplication.translate("MainWindow", "Cell models count", None, -1))
        self.labelModelsCnt.setText(QtWidgets.QApplication.translate("MainWindow", "0", None, -1))
        self.labelTargetCnt.setToolTip(QtWidgets.QApplication.translate("MainWindow", "Loaded target genes count", None, -1))
        self.labelTargetCnt.setStatusTip(QtWidgets.QApplication.translate("MainWindow", "Loaded target genes count", None, -1))
        self.labelTargetCnt.setText(QtWidgets.QApplication.translate("MainWindow", "0", None, -1))
        self.buttonShowTarget.setText(QtWidgets.QApplication.translate("MainWindow", "Show Table", None, -1))
        self.label_2.setToolTip(QtWidgets.QApplication.translate("MainWindow", "Table of target genes loaded into the program", None, -1))
        self.label_2.setStatusTip(QtWidgets.QApplication.translate("MainWindow", " Table of target genes loaded into the program", None, -1))
        self.label_2.setText(QtWidgets.QApplication.translate("MainWindow", "Target genes:", None, -1))
        self.labelRefCnt.setToolTip(QtWidgets.QApplication.translate("MainWindow", "Loaded reference genes cout", None, -1))
        self.labelRefCnt.setStatusTip(QtWidgets.QApplication.translate("MainWindow", "Loaded reference genes cout", None, -1))
        self.labelRefCnt.setText(QtWidgets.QApplication.translate("MainWindow", "0", None, -1))
        self.label_3.setToolTip(QtWidgets.QApplication.translate("MainWindow", "A current list of cell line models", None, -1))
        self.label_3.setStatusTip(QtWidgets.QApplication.translate("MainWindow", "A current list of cell line models", None, -1))
        self.label_3.setText(QtWidgets.QApplication.translate("MainWindow", "Line models:", None, -1))
        self.buttonShowModels.setText(QtWidgets.QApplication.translate("MainWindow", "Show List", None, -1))
        self.label.setToolTip(QtWidgets.QApplication.translate("MainWindow", "Table of reference genes loaded into the program", None, -1))
        self.label.setStatusTip(QtWidgets.QApplication.translate("MainWindow", "Table of reference genes loaded into the program", None, -1))
        self.label.setText(QtWidgets.QApplication.translate("MainWindow", "Reference gens:", None, -1))
        self.label_4.setText(QtWidgets.QApplication.translate("MainWindow", "Quantitive data:", None, -1))
        self.labelQantitive.setText(QtWidgets.QApplication.translate("MainWindow", "Not loaded", None, -1))
        self.buttonRemoveQuantitive.setText(QtWidgets.QApplication.translate("MainWindow", "Remove", None, -1))
        self.groupBox_2.setTitle(QtWidgets.QApplication.translate("MainWindow", "Parameters", None, -1))
        self.label_8.setToolTip(QtWidgets.QApplication.translate("MainWindow", "Select normalization algorithm to use during analysis ", None, -1))
        self.label_8.setStatusTip(QtWidgets.QApplication.translate("MainWindow", "Select normalization algorithm to use during analysis ", None, -1))
        self.label_8.setText(QtWidgets.QApplication.translate("MainWindow", "Normalization algorithm", None, -1))
        self.comboBoxNormalizationAlgorithm.setItemText(0, QtWidgets.QApplication.translate("MainWindow", "Normfinder", None, -1))
        self.label_5.setToolTip(QtWidgets.QApplication.translate("MainWindow", "Select statistical model to use during analysis ", None, -1))
        self.label_5.setStatusTip(QtWidgets.QApplication.translate("MainWindow", "Select statistical model to use during analysis ", None, -1))
        self.label_5.setText(QtWidgets.QApplication.translate("MainWindow", "Statistical model:", None, -1))
        self.comboBoxStatModel.setItemText(0, QtWidgets.QApplication.translate("MainWindow", "Student", None, -1))
        self.label_6.setToolTip(QtWidgets.QApplication.translate("MainWindow", "Select p-value", None, -1))
        self.label_6.setStatusTip(QtWidgets.QApplication.translate("MainWindow", "Select p-value", None, -1))
        self.label_6.setText(QtWidgets.QApplication.translate("MainWindow", "confidence:", None, -1))
        self.lineEditConfidence.setText(QtWidgets.QApplication.translate("MainWindow", "0.05", None, -1))
        self.label_7.setToolTip(QtWidgets.QApplication.translate("MainWindow", "Select number of remove worst reference gene cycles", None, -1))
        self.label_7.setStatusTip(QtWidgets.QApplication.translate("MainWindow", "Select number of remove worst reference gene cycles", None, -1))
        self.label_7.setText(QtWidgets.QApplication.translate("MainWindow", "Remove repetitions:", None, -1))
        self.lineEditRemoveRep.setText(QtWidgets.QApplication.translate("MainWindow", "2", None, -1))
        self.label_9.setToolTip(QtWidgets.QApplication.translate("MainWindow", "Select check repetition for a single sample", None, -1))
        self.label_9.setStatusTip(QtWidgets.QApplication.translate("MainWindow", "Select check repetition for a single sample", None, -1))
        self.label_9.setText(QtWidgets.QApplication.translate("MainWindow", "Single sample repetition", None, -1))
        self.lineEditSampleRepetition.setText(QtWidgets.QApplication.translate("MainWindow", "3", None, -1))
        self.checkSelectBest.setText(QtWidgets.QApplication.translate("MainWindow", "Select best remove for model", None, -1))
        self.buttonRunCalculations.setText(QtWidgets.QApplication.translate("MainWindow", "Run calculations", None, -1))
        self.groupBoxResultsSummary.setTitle(QtWidgets.QApplication.translate("MainWindow", "Results summary", None, -1))
        self.label_17.setText(QtWidgets.QApplication.translate("MainWindow", "Coherence score list", None, -1))
        self.buttonExportResults.setText(QtWidgets.QApplication.translate("MainWindow", "Export results", None, -1))
        self.buttonExportGraphs.setText(QtWidgets.QApplication.translate("MainWindow", "Export graphs", None, -1))
        self.buttonShowBestRef.setText(QtWidgets.QApplication.translate("MainWindow", "Show best references", None, -1))
        self.buttonShowRQ.setText(QtWidgets.QApplication.translate("MainWindow", "Show RQ values", None, -1))
        self.buttonShowStatistic.setText(QtWidgets.QApplication.translate("MainWindow", "Show p-values", None, -1))
        self.menuFile.setTitle(QtWidgets.QApplication.translate("MainWindow", "File", None, -1))
        self.actionRead_from_combined_input.setText(QtWidgets.QApplication.translate("MainWindow", "Read from combined input", None, -1))
        self.actionLoad_Quantified.setText(QtWidgets.QApplication.translate("MainWindow", "Load Quantified", None, -1))
        self.actionActivate_license.setText(QtWidgets.QApplication.translate("MainWindow", "Activate license", None, -1))




from itertools import combinations

from genexpa.uis.uis.ReadFromCombinedDialogUI import Ui_Dialog
from PySide2.QtWidgets import QDialog, QFileDialog
from PySide2 import QtCore

from genexpa.uis import LineModelDialog
from genexpa.core import helpers
from genexpa.core import explib


class PandasModel(QtCore.QAbstractTableModel):
    def __init__(self, df, parent=None):
        QtCore.QAbstractTableModel.__init__(self, parent=parent)
        self._df = df.copy()

    def toDataFrame(self):
        return self._df.copy()

    def headerData(self, section, orientation, role=QtCore.Qt.DisplayRole):
        if role != QtCore.Qt.DisplayRole:
            return None

        if orientation == QtCore.Qt.Horizontal:
            return self._df.columns.tolist()[section]

        elif orientation == QtCore.Qt.Vertical:
            return self._df.index.tolist()[section]

    def data(self, index, role=QtCore.Qt.DisplayRole):
        if role != QtCore.Qt.DisplayRole:
            return None

        if not index.isValid():
            return None

        val = self._df.iloc[index.row(), index.column()]
        return str(val)

    def setData(self, index, value, role):
        row = self._df.index[index.row()]
        col = self._df.columns[index.column()]
        if hasattr(value, 'toPyObject'):
            # PyQt4 gets a QVariant
            value = value.toPyObject()
        else:
            # PySide gets an unicode
            dtype = self._df[col].dtype
            if dtype != object:
                value = None if value == '' else dtype.type(value)
        self._df.set_value(row, col, value)
        return True

    def rowCount(self, parent=QtCore.QModelIndex()):
        return len(self._df.index)

    def columnCount(self, parent=QtCore.QModelIndex()):
        return len(self._df.columns)

class GenesModel(QtCore.QAbstractListModel):
    def __init__(self, genes, sel_inds=None):
        super().__init__()
        self.genes = genes
        if sel_inds is None:
            self.active_inds = list(range(len(self.genes)))
        else:
            self.active_inds = sel_inds

    def data(self, index, role):
        if role == QtCore.Qt.DisplayRole:
            return self.genes[self.active_inds[index.row()]]

    def rowCount(self, index):
        return len(self.active_inds)

    def addGene(self, ind):
        if ind > len(self.genes):
            return None

        if ind in self.active_inds:
            return None

        insert_ind = None
        for i, active_ind in enumerate(self.active_inds):
            if active_ind > ind:
                insert_ind = i
                break

        if insert_ind is None:
            insert_ind = len(self.active_inds)

        self.beginInsertRows(QtCore.QModelIndex(), insert_ind, insert_ind)
        self.active_inds.insert(insert_ind, ind)
        self.endInsertRows()

    def removeGene(self, ind):
        if ind not in self.active_inds:
            return  None

        remove_ind = self.active_inds.index(ind)
        self.beginRemoveRows(QtCore.QModelIndex(),remove_ind, remove_ind)
        self.active_inds.pop(remove_ind)
        self.endRemoveRows()

    def removeInd(self, remove_ind):
        if remove_ind > len(self.active_inds):
            return None

        self.beginRemoveRows(QtCore.QModelIndex(),remove_ind, remove_ind)
        self.active_inds.pop(remove_ind)
        self.endRemoveRows()

    def getActiveGenes(self):
        return [self.genes[i] for i in self.active_inds]

class LinesModel(QtCore.QAbstractListModel):
    def __init__(self):
        super().__init__()
        self.lines_models = []

    def data(self, index, role):
        if role == QtCore.Qt.DisplayRole:
            lines = self.lines_models[index.row()]
            lines_str = " ".join(lines)
            return lines_str

    def rowCount(self, index):
        return len(self.lines_models)

    def addLineModel(self, new_line_model):
        if new_line_model is None or new_line_model == []:
            return  None
        new_line_model_set = set(new_line_model)

        for line_model in self.lines_models:
            if set(line_model) == new_line_model_set:
                return None

        if len(new_line_model) < 2:
            return None

        end_ind = len(self.lines_models)
        self.beginInsertRows(QtCore.QModelIndex(), end_ind, end_ind)
        self.lines_models.append(tuple(new_line_model))
        self.endInsertRows()

    def removeLineModel(self, ind):
        if ind > len(self.lines_models):
            return None

        self.beginRemoveRows(QtCore.QModelIndex(), ind, ind)
        self.lines_models.pop(ind)
        self.endRemoveRows()

    def clearModel(self):
        if len(self.lines_models) == 0:
            return None

        self.beginRemoveRows(QtCore.QModelIndex(), 0, len(self.lines_models)-1)
        self.lines_models.clear()
        self.endRemoveRows()

    def getCurrentLines(self):
        return self.lines_models[:]

class ReadFromCombinedDialog(QDialog):
    def __init__(self, parent, actions, system):
        super().__init__(parent)
        self.actions = actions
        self.system = system

        self.ui = Ui_Dialog()
        self.ui.setupUi(self)

        self.loaded_df = None
        self.loaded_df_model = None

        self.cell_lines = []

        self.init_connections()

    def init_connections(self):
        self.ui.buttonSelectFile.clicked.connect(self.read_file)

        self.ui.buttonSelectRef.clicked.connect(self.add_ref_gene)
        self.ui.buttonUnSelectRef.clicked.connect(self.remove_ref_gene)
        self.ui.buttonAddSingle.clicked.connect(self.add_line_model)

        self.ui.buttonDeleteModel.clicked.connect(self.remove_line_model)
        self.ui.buttonRemoveAll.clicked.connect(self.clear_line_model)
        self.ui.buttonGenerateComb.clicked.connect(self.generate_comb_line_model)

        self.ui.buttonAccept.clicked.connect(self.accept)
        self.ui.buttonCancel.clicked.connect(self.close)

    def read_file(self):
        filename, filter = QFileDialog.getOpenFileName(
            parent=self.system.mainWindowApp,
            caption='Open combined input file',
            dir='.',
            filter='*.xlsx *.xls')

        if filename == "":
            return None

        df = explib.read_combined_input(filename)

        self.ui.loadedFileText.setText(filename)
        self.set_loaded_df(df)

    def set_loaded_df(self, df):
        self.loaded_df = df
        self.loaded_df_model = PandasModel(self.loaded_df)
        self.ui.tableLoadedDF.setModel(self.loaded_df_model)

        # update other models
        self.avail_genes = self.loaded_df.index.unique().tolist()

        self.avail_genes_model = GenesModel(self.avail_genes)
        self.ui.listAvailRef.setModel(self.avail_genes_model)

        self.selected_genes_model = GenesModel(self.avail_genes, [])
        self.ui.listSelRef.setModel(self.selected_genes_model)

        self.cell_lines_model = LinesModel()
        self.ui.listCurrentModels.setModel(self.cell_lines_model)

        self.cell_lines = self.loaded_df.columns.unique().tolist()

    def add_ref_gene(self):
        for qmodel in self.ui.listAvailRef.selectedIndexes():
            self.selected_genes_model.addGene(qmodel.row())

    def remove_ref_gene(self):
        for qmodel in self.ui.listSelRef.selectedIndexes():
            self.selected_genes_model.removeInd(qmodel.row())

    def add_line_model(self):
        lines_selector_dialog = LineModelDialog(self.system.mainWindowApp,
             self.cell_lines)

        accepted = lines_selector_dialog.exec_()
        if not accepted:
            return None

        self.cell_lines_model.addLineModel(lines_selector_dialog.selected_lines)

    def remove_line_model(self):
        for qmodel in self.ui.listCurrentModels.selectedIndexes():
            self.cell_lines_model.removeLineModel(qmodel.row())

    def clear_line_model(self):
        self.cell_lines_model.clearModel()

    def generate_comb_line_model(self):
        for r in range(2, len(self.cell_lines)+1):
            for comb in combinations(self.cell_lines, r):
                self.cell_lines_model.addLineModel(comb)


    def get_dataframes(self):
        # return df_ref, df_target, line_models
        try:
            ref_genes =  self.selected_genes_model.getActiveGenes()
            target_genes = [gene for gene in self.selected_genes_model.genes if gene not in ref_genes]

            df_ref = self.loaded_df.loc[ref_genes,:].copy()
            df_target = self.loaded_df.loc[target_genes,:].copy()

            lines_models = self.cell_lines_model.getCurrentLines()

            return df_ref, df_target, lines_models
        except:
            return None



import pandas as pd
import numpy as np
import os

import scikit_posthocs
from scipy import stats

import itertools

import matplotlib.pyplot as plt
import matplotlib.patches as patches
from matplotlib.font_manager import FontProperties

from genexpa.core.normfinder import normfinder

def clean_name(name):
    # repalce dividing character for safety
    return name.replace("|", "_")

def sort_clean_grups(glist):
    cleaned = [clean_name(gr) for gr in glist]
    return sorted(cleaned)

def join_tuples(df, row=None, col=None):
    cp = df.copy()
    if row is not None:
        cp.index = cp.index.set_levels(cp.index.levels[row].map(" ".join), level=row)

    if col is not None:
        cp.columns = cp.columns.set_levels(cp.columns.levels[col].map(" ".join), level=col)

    return cp

def create_stat_matrix(pval, columns):
    vals = np.full([len(columns), len(columns)], pval)
    np.fill_diagonal(vals, 1.0)

    df = pd.DataFrame(vals, columns=columns, index=columns)
    return df
# ------

def full_excel_export(output_file, df_list):
    mean_df = join_tuples(df_list[0], row=0)
    best_ref_df = join_tuples(df_list[1], row=0)
    RQ_df = join_tuples(df_list[2], row=0)
    stat_df = join_tuples(df_list[3], row=0, col=1)
    comparison_df = join_tuples(df_list[4], row=0, col=1)
    coherence_df = join_tuples(df_list[5], col=1)

    dfs = [mean_df, best_ref_df, RQ_df, stat_df, comparison_df, coherence_df]
    names = ["mean RQ values", "best reference genes", "RQ values", "statistic", "comparison", "partial coherence"]

    writer = pd.ExcelWriter(output_file)
    for name, df in zip(names, dfs):
        df.to_excel(writer, name)
    writer.save()

def read_combined_input(ifile, format='excel', sep=","):
    """
    read combined data file in format:
           Line_1  Line_2  Line_3 ...
    GeneA    x        x      x
    GeneA    x        x      x
    ...
    GeneB    x        x      x
    ...

    and converting it into library internal format:
            Line_1  Line_1 Line_1 ... Line_2 ...
    GeneA     x       x      x          x
    GeneB     x       x      x          x
    ...
    """
    if format == 'csv':
        res = pd.read_csv(ifile, sep=sep, index_col=0)
    elif format == 'excel':
        res = pd.read_excel(ifile, index_col=0)
    else:
        ValueError("format: {}, is not supported!".format(format))

    res.index.name = "Gene"

    basedf = None
    for gene_name, _df in res.groupby("Gene"):
        gene_line = []
        column_name_line = []

        for col in _df.columns:
            expression_values = _df[col].values.T
            column_name_line.extend([col]*len(expression_values))
            gene_line.extend(expression_values)

        df_row = pd.DataFrame(data=[gene_line], columns=column_name_line)
        df_row.index = [gene_name]

        if basedf is not None:
            basedf = pd.concat([basedf, df_row])
        else:
            basedf = df_row

    return basedf

def append_group_index(df_ref):
    """
    append unique index for each cell lines (starting from 1)
    """
    group_int_code, _ = pd.factorize(pd.Series(df_ref.columns))
    group_int_code += 1

    group_df = pd.DataFrame(data=[group_int_code], columns=df_ref.columns)
    group_df.index = ["group"]

    return pd.concat([df_ref, group_df])

def flatten_mlvl_index(df, sep='_', arr_sep="|"):
    _df = df.copy()

    if isinstance(df.index, pd.MultiIndex):
        new_rows = []
        for row in df.index.values:
            new_row = []
            for row_value in row:
                if type(row_value) in [list, set, tuple, np.ndarray]:
                    new_row.append(arr_sep.join(row_value))
                else:
                    new_row.append(str(row_value))

            new_rows.append(sep.join(new_row))

        _df.index = pd.Index(new_rows)

    if  isinstance(df.columns, pd.MultiIndex):
        new_rows = []
        for row in df.columns.values:
            new_row = []
            for row_value in row:
                if type(row_value) in [list, set, tuple, np.ndarray]:
                    new_row.append(arr_sep.join(row_value))
                else:
                    new_row.append(str(row_value))

            new_rows.append(sep.join(new_row))

        _df.columns = pd.Index(new_rows)

    return _df

def calc_best_ref(df_ref, df_ref_norm, single_sample_rep, norm_algo, pair_sep=" "):
    """
    calculation of ref Ct values and returning best gene or combination
    """

    if df_ref_norm is None:
        best_genes, best_pairs = normfinder(df_ref)
    else:
        best_genes, best_pairs = normfinder(df_ref_norm.loc[df_ref.index][df_ref.columns.unique()])

    if best_genes[0][1] > best_pairs[0][2]:
        stability_value = best_pairs[0][2]
        best_gene1 = best_pairs[0][0]
        best_gene2 = best_pairs[0][1]
        best_gene = best_gene1 + pair_sep + best_gene2

        best_gene_arr1 = df_ref.loc[best_gene1].values
        best_gene_arr2 = df_ref.loc[best_gene2].values
        best_gene_arr = (best_gene_arr1 + best_gene_arr2)/2.0
    else:
        stability_value = best_genes[0][1]
        best_gene = best_genes[0][0]
        best_gene_arr = df_ref.loc[best_gene].values

    # single sample average over single_sample_rep
    sep_arr = best_gene_arr.reshape(-1, single_sample_rep)
    sep_mean_arr = np.mean(sep_arr, axis=1)
    mean_ref_arr = np.tile(sep_mean_arr.reshape(-1,1), single_sample_rep).reshape(-1)

    df_best_ref = pd.DataFrame(data=[mean_ref_arr], columns=df_ref.columns)
    df_best_ref.index = [best_gene]

    return df_best_ref, stability_value

def remove_worst_ref(df_ref, remove, df_ref_norm, norm_algo):
    if remove == 0:
        return df_ref

    if df_ref_norm is None:
        best_genes, best_pairs = normfinder(df_ref)
    else:
        best_genes, best_pairs = normfinder(df_ref_norm.loc[df_ref.index][df_ref.columns.unique()])

    worst_gene = best_genes[-1][0]

    df_trimed = df_ref[df_ref.index != worst_gene].copy()

    if remove > 1:
        df_trimed = remove_worst_ref(df_trimed, remove-1, df_ref_norm, norm_algo)

    return df_trimed

def calc_RQ(df_target, df_best_ref):
    delta_ct = df_target - df_best_ref.values.squeeze()
    return delta_ct.apply(lambda x: np.power(2,-x))

# ---------------------------------------------------
def stat_analyze(RQ, lines, alpha, mode):
    # p2_cut - The two-tailed p-value.
    # mode ["Kruskal-Wallis H-test", "Kolmogorov-Smirnov t-test", "Welch's t-test"]
    # return dictionary with analyze results, key - gene
    stat_dict = {}
    for gene in RQ.index:
        result_dict = {}
        means = {}
        for line in lines:
            # transpose for better analysis
            result_dict[line] = RQ.loc[gene,line].values
            means[line] = np.mean(RQ.loc[gene,line].values)

        df = pd.DataFrame(result_dict).dropna(1)
        active_lines = len(df.columns)

        pair_stat = [] # all gene comb
        if mode == 0: # Pairwise t-test, Holm adjustment
            if active_lines < 2:
                stat_matrix = pd.DataFrame()
            else:
                stat_matrix = scikit_posthocs.posthoc_ttest(df.melt(), group_col='variable', val_col='value', p_adjust='holm')

        elif mode == 1: # Kolmogorov-Smirnov / Kruskal-Wallis -> post Dunn's test
            if active_lines < 2:
                stat_matrix = pd.DataFrame()
            elif active_lines == 2:
                pval = stats.ks_2samp(*df.values.T).pvalue
                stat_matrix = create_stat_matrix(pval, df.columns)

            else:
                group_pval = stats.kruskal(*df.values.T).pvalue
                if group_pval < alpha:
                    stat_matrix = scikit_posthocs.posthoc_dunn(df.melt(), group_col='variable', val_col='value',
                                                               p_adjust="bonferroni")
                else:
                    stat_matrix = create_stat_matrix(group_pval, df.columns)

        elif mode == 2: # Mann-Whitney / Kruskal-Wallis -> post Dunn's test
            if active_lines < 2:
                stat_matrix = pd.DataFrame()
            elif active_lines == 2:
                pval = stats.mannwhitneyu(*df.values.T).pvalue
                stat_matrix = create_stat_matrix(pval, df.columns)

            else:
                group_pval = stats.kruskal(*df.values.T).pvalue
                if group_pval < alpha:
                    stat_matrix = scikit_posthocs.posthoc_dunn(df.melt(), group_col='variable', val_col='value',
                                                               p_adjust="bonferroni")
                else:
                    stat_matrix = create_stat_matrix(group_pval, df.columns)

        elif mode == 3: # Kruskal-Wallis
            if active_lines < 2:
                stat_matrix = pd.DataFrame()
            else:
                group_pval = stats.kruskal(*df.values.T).pvalue
                if group_pval < alpha:
                    stat_matrix = create_stat_matrix(1.0, df.columns)
                    for line_a, line_b in itertools.combinations(df.columns, 2):
                        pval = stats.kruskal(*df[[line_a, line_b]].values.T).pvalue
                        stat_matrix.loc[line_a, line_b] = pval
                        stat_matrix.loc[line_b, line_a] = pval
                else:
                    stat_matrix = create_stat_matrix(group_pval, df.columns)

        for line_a, line_b in itertools.combinations(lines, 2):
            if line_a in stat_matrix.index and line_b in stat_matrix.index:
                p_val = stat_matrix.loc[line_a][line_b]
            else:
                p_val = np.nan

            pair_stat.append([line_a, line_b, means[line_a], means[line_b], p_val])

        stat_dict[gene] = pair_stat

    return stat_dict

def calc_coherence(comparison_df):
    coherence_line = []
    for gene, pair in comparison_df.columns:
        value_list = comparison_df.loc[:, gene][pair].dropna()
        value_set = set(value_list)

        if 1 in value_set and -1 in value_set:
            coherence_line.append(0)

        else:
            coherence_line.append(1)

    coherence_df = pd.DataFrame(
        data=[coherence_line],
        index=["coherence"],
        columns=comparison_df.columns
    )

    return coherence_df

def full_analyze(df_ref, df_target, models, remove_num, single_sample_rep, stat_mode, alpha, df_ref_norm, norm_algo, sort_lines=True):
    target_genes = df_target.index.tolist()

    mean_df_list = []
    RQ_df_list = []
    stat_df_list = []
    comparison_df_list = []

    best_ref_df_list = []
    best_ref_col_index = ["best", "value"]

    if df_ref_norm is not None:
        df_ref_norm = pd.concat([df_ref_norm, df_ref.loc[["group"]]])

    for model in models:
        df_sel_ref = df_ref.loc[:,model]
        df_sel_target = df_target.loc[:,model]

        best_ref_rm_list = []
        mean_rm_list = []
        RQ_rm_list = []
        stat_rm_list = []
        comparison_rm_list = []

        for rm_num in range(remove_num+1):
            local_df_sel_ref = remove_worst_ref(df_sel_ref, rm_num, df_ref_norm, norm_algo) # explib
            df_best_ref, stability_value = calc_best_ref(local_df_sel_ref, df_ref_norm, single_sample_rep, norm_algo) # explib

            best_ref_gene = df_best_ref.index[0]
            best_ref_rm_list.append([best_ref_gene, stability_value])

            RQ = calc_RQ(df_sel_target, df_best_ref) # explib
            gene_stat_dict = stat_analyze(RQ, model, alpha, stat_mode) # explib


            mean_local = []
            RQ_local = []
            stat_local = []
            comparison_local = []
            for target_gene in target_genes:
                mean_helper_dict = {}
                for gr1, gr2, m1, m2, pval in gene_stat_dict[target_gene]:
                    mean_helper_dict[gr1] = m1
                    mean_helper_dict[gr2] = m2

                    stat_local.append(pval)
                    if np.isnan(m1) or np.isnan(m2):
                        comparison_local.append(np.nan)
                        continue

                    if pval < alpha:
                        if m1 > m2:
                            comparison_local.append(1)
                        else:
                            comparison_local.append(-1)

                    else:
                        comparison_local.append(0)

                mean_local.extend([mean_helper_dict[key] for key in model])

                for key in model:
                    RQ_local.append(RQ.loc[target_gene][key].values.reshape(-1,1))

            mean_rm_list.append(mean_local)
            RQ_rm_list.append(np.concatenate(RQ_local, axis=1))
            stat_rm_list.append(stat_local)
            comparison_rm_list.append(comparison_local)

        # BEST REF DF
        model_str = "|".join(model)
        row_mlvl_index = pd.MultiIndex.from_product(
            [[model],list(range(remove_num+1))], names=["model", 'removal'])

        _df_best = pd.DataFrame(
            data=best_ref_rm_list,
            index=row_mlvl_index,
            columns=best_ref_col_index)
        best_ref_df_list.append(_df_best)

        # MEAN DF
        mean_col_mlvl_index =  pd.MultiIndex.from_product(
            [target_genes, model], names=['target', 'group'])
        _mean_df = pd.DataFrame(
            data=mean_rm_list,
            index=row_mlvl_index,
            columns=mean_col_mlvl_index)
        mean_df_list.append(_mean_df)

        # RQ DF
        RQ_data = np.concatenate(RQ_rm_list)
        RQ_row_mlvl_index = pd.MultiIndex.from_product(
            [[model],
             list(range(remove_num+1)),
             range(RQ_data.shape[0]//(remove_num+1))], names=["model", 'removal', 'measurment'])

        _RQ_df = pd.DataFrame(
            data=RQ_data,
            index=RQ_row_mlvl_index,
            columns=mean_col_mlvl_index)
        RQ_df_list.append(_RQ_df)

        # stat DF
        model_pair = list(itertools.combinations(model, 2))
        stat_col_mlvl_index = pd.MultiIndex.from_product(
            [target_genes, model_pair],
            names=['target', 'pair'])
        _stat_df = pd.DataFrame(
            data=stat_rm_list,
            index=row_mlvl_index,
            columns=stat_col_mlvl_index)
        stat_df_list.append(_stat_df)

        # comparison DF
        _comparison_df = pd.DataFrame(
            data=comparison_rm_list,
            index=row_mlvl_index,
            columns=stat_col_mlvl_index)
        comparison_df_list.append(_comparison_df)

    best_ref_df = pd.concat(best_ref_df_list)
    mean_df = pd.concat(mean_df_list)
    RQ_df = pd.concat(RQ_df_list)

    stat_df = pd.concat(stat_df_list)
    stat_df.stat_model = stat_mode
    stat_df.alpha = alpha

    comparison_df = pd.concat(comparison_df_list)

    if sort_lines:
        # prepare new column order for dataframes
        models_lengths = [len(_m) for _m in models]
        final_model = list(models[np.argsort(models_lengths)[-1]])

        unique_lines = list(set([item for sublist in models for item in sublist]))

        for _line in unique_lines:
            if _line not in final_model:
                final_model.append(_line)

        _dfs = []
        _keys = []

        for _gene in mean_df.columns.levels[0]:
            _dfs.append(mean_df[_gene][final_model])
            _keys.append(_gene)
        mean_df = pd.concat(_dfs, axis=1, keys=_keys)

        _dfs = []
        _keys = []

        for _gene in RQ_df.columns.levels[0]:
            _dfs.append(RQ_df[_gene][final_model])
            _keys.append(_gene)
        RQ_df = pd.concat(_dfs, axis=1, keys=_keys)

    return mean_df, best_ref_df, RQ_df, stat_df, comparison_df

def get_best_series_remove(best_ref_df):
    mask = []
    for model in best_ref_df.index.get_level_values(0).unique():
        model_df = best_ref_df.loc[model]

        _mask = np.zeros(model_df.shape[0], bool)

        _mask[model_df.value.values.argmin()] = True
        mask.extend(_mask)

    return np.array(mask)

def expand_mask(mask, df):
    full_mask = []
    i = 0
    for model in df.index.get_level_values(0).unique():
        for rm_lvl in df.loc[model].index.get_level_values(0).unique():
            samples = df.loc[model].loc[rm_lvl].shape[0]
            full_mask.extend([mask[i]]*samples)
            i+=1
            
    return np.array(full_mask)

def write_excel(df, opath):
    # Create a Pandas Excel writer using XlsxWriter as the engine. '/tmp/pandas_simple.xlsx'
    writer = pd.ExcelWriter(opath, engine='xlsxwriter')

    # Convert the dataframe to an XlsxWriter Excel object.
    df.to_excel(writer, sheet_name='Sheet1')

    # Close the Pandas Excel writer and output the Excel file.
    writer.save()

def print_combination(res, alpha, single=None, retstr=False):
    ostr = ""
    for gene in res:
        if single:
            if gene != single:
                continue
        ostr += "----------------------\n{}\n----------------------".format(gene)
        ostr += ("\n{:<20}{:<3}{:<3}{:<3}".format("Pair", "g1", "g2", "ins"))

        for pair in res[gene]:
            gr1_larger = 0
            gr2_larger = 0
            insig = 0
            for case in res[gene][pair]:
                if case[2] > alpha:
                    insig += 1
                elif case[0] > case[1]:
                    gr1_larger += 1
                else:
                    gr2_larger += 1
            pair_str = "{:<10}{:<10}".format(pair[0], pair[1])
            ostr += "\n{:<20}{:<3}{:<3}{:<3}".format(pair_str, gr1_larger, gr2_larger, insig)

        ostr += "\n"
    if retstr:
        return ostr

def plot_gene_analyze(pair_stat, df, ax=None, gene_name=""):
    if ax is not None:
        bplot = df.boxplot(column=list(df.columns), ax=ax)
    else:
        fig, ax = plt.subplots(1,1, figsize=(16,16))
        bplot = df.boxplot(column=list(df.columns), ax=ax)

    props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)

    pair_stat = sorted(pair_stat, key=lambda x: x[2])
    box_text = "  The two-tailed p-value:\n"
    box_text += "\n".join( "{a[0]: <10}{a[1]: <10} = {a[2]:.2e}".format(a=x) for x in pair_stat)

    # add place on y for text
    y_min, y_max = ax.get_ylim()
    y_max = y_max+(y_max-y_min)*0.4
    ax.set_ylim([y_min, y_max])

    #font.set_family("monospace")
    font = FontProperties()
    font.set_family("monospace")
    ax.set_title(gene_name)
    ax.text(0.02, 0.98, box_text, transform=ax.transAxes, bbox=props, verticalalignment='top',
           fontproperties=font)

def box_plot(RQ, df_stat, model, title="", savefig=None, alpha=0.05, line_color='#ff1b00ff', axsize=(6,6), figsize=None, columns=2, line_width_frac=0.01, interline=2):
    RQ = RQ.dropna(1)
    df_stat = df_stat.dropna(0)

    genes = RQ.columns.get_level_values(0).unique().tolist()
    genes_cnt = len(genes)

    rows = int(np.ceil(genes_cnt/float(columns)))
    if figsize is None:
        figsize=(axsize[0]*columns, axsize[1]*rows)


    fig, axes = plt.subplots(rows, columns, figsize=figsize)
    fig.suptitle(title)

    i = 0
    for gene in genes:
        if rows == 1:
            ax = axes[i]
        else:
            ax = axes[i//columns][i%columns]

        ax.set_title(gene)

        valid_model = []
        for _line in model:
            if _line in RQ[gene].columns:
                valid_model.append(_line)

        if gene not in df_stat:
            continue

        RQ_sel = RQ[gene]
        if len(RQ_sel.columns) < 2:
            continue
        RQ_sel.boxplot(column=valid_model, ax=ax)
        x_group = dict(zip(valid_model, range(1,len(valid_model)+1)))

        ymin, ymax = ax.get_ylim()
        ax_height = ymax-ymin

        y_current = ymin

        line_width = line_width_frac*ax_height
        
        # draw connections

        for pair, value in df_stat[gene].iteritems():
            if type(alpha) is str:
                continue

            if value > alpha:
                continue

            left_group, right_group = pair
            if right_group not in x_group or left_group not in x_group:
                continue

            x_start = x_group[left_group]
            x_len = x_group[right_group]-x_group[left_group]

            ax.add_patch(
                patches.Rectangle((x_start, y_current), x_len, line_width, facecolor=line_color, zorder=-100)
            )
            y_current -= interline*line_width

        ax.set_ylim(y_current, ymax)

        i += 1

    axes_flat = axes.flatten()
    for rm_ax in axes_flat[i:]:
        fig.delaxes(rm_ax)

    if savefig:
        fig.savefig(savefig)
        fig.clear()
        plt.close()

def create_boxplots(RQ, df_stat, base):
    try:
        os.mkdir(base)
    except:
        pass

    if hasattr(df_stat, "alpha"):
        alpha = df_stat.alpha
    else:
        alpha = "None"

    if hasattr(df_stat, "stat_model"):
        stat_model = df_stat.stat_model
    else:
        stat_model = "None"

    models = RQ.index.get_level_values(0).unique().tolist()

    for model in models:
        rmlvls = RQ.loc[model].index.get_level_values(0).unique().values
        for rmlvl in rmlvls:
            name = " ".join(model)
            title = "{}\nmodel={},\nalpha={}, removed={}".format(name, stat_model, alpha, rmlvl)
            _RQ = RQ.loc[model].loc[rmlvl]
            _df_stat = df_stat.loc[model].loc[rmlvl]

            save_dir = os.path.join(base, "model_{}__removed_{}.png".format(name, rmlvl))
            box_plot(_RQ, _df_stat, model = model, title=title, savefig=save_dir, alpha=alpha)



from genexpa.uis.uis.LineModelDialogUI import Ui_Dialog
from PySide2.QtWidgets import QDialog
from PySide2 import QtCore


class LineListModel(QtCore.QAbstractListModel):
    def __init__(self, lines):
        super().__init__()
        self.lines = lines

    def data(self, index, role):
        if role == QtCore.Qt.DisplayRole:
            return self.lines[index.row()]

    def rowCount(self, index):
        return len(self.lines)

class LineModelDialog(QDialog):
    def __init__(self, parent, lines):
        super().__init__(parent)

        self.ui = Ui_Dialog()
        self.ui.setupUi(self)

        self.selected_lines = []
        self.selected_lines_str = ""

        self.lines = lines
        self.list_model = LineListModel(self.lines)
        self.ui.listLines.setModel(self.list_model)

        self.init_connections()

    def init_connections(self):
        self.ui.buttonAddLine.clicked.connect(self.add_to_line)
        self.ui.buttonDiscard.clicked.connect(self.close)
        self.ui.buttonAccept.clicked.connect(self.accept)

    def add_to_line(self):
        for qmodel in self.ui.listLines.selectedIndexes():
            ind = qmodel.row()
            sel_line = self.lines[ind]
            if sel_line not in self.selected_lines:
                self.add_to_current_model(sel_line)


    def add_to_current_model(self, line):
        self.selected_lines.append(line)
        self.selected_lines_str += " {}".format(line)
        self.ui.labelModelText.setText(self.selected_lines_str)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ReadFromCombinedDialog.ui',
# licensing of 'ReadFromCombinedDialog.ui' applies.
#
# Created: Sat Jan  4 12:25:52 2020
#      by: pyside2-uic  running on PySide2 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.setWindowModality(QtCore.Qt.ApplicationModal)
        Dialog.resize(860, 666)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBoxLoadFile = QtWidgets.QGroupBox(Dialog)
        self.groupBoxLoadFile.setMinimumSize(QtCore.QSize(0, 100))
        self.groupBoxLoadFile.setObjectName("groupBoxLoadFile")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBoxLoadFile)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.groupBoxLoadFile)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.loadedFileText = QtWidgets.QLabel(self.groupBoxLoadFile)
        self.loadedFileText.setObjectName("loadedFileText")
        self.horizontalLayout.addWidget(self.loadedFileText)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.tableLoadedDF = QtWidgets.QTableView(self.groupBoxLoadFile)
        self.tableLoadedDF.setObjectName("tableLoadedDF")
        self.horizontalLayout_2.addWidget(self.tableLoadedDF)
        self.widget = QtWidgets.QWidget(self.groupBoxLoadFile)
        self.widget.setMinimumSize(QtCore.QSize(180, 0))
        self.widget.setObjectName("widget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.buttonSelectFile = QtWidgets.QPushButton(self.widget)
        self.buttonSelectFile.setObjectName("buttonSelectFile")
        self.verticalLayout_2.addWidget(self.buttonSelectFile)
        self.horizontalLayout_2.addWidget(self.widget)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.verticalLayout.addWidget(self.groupBoxLoadFile)
        self.groupBoxSelectRef = QtWidgets.QGroupBox(Dialog)
        self.groupBoxSelectRef.setMinimumSize(QtCore.QSize(0, 100))
        self.groupBoxSelectRef.setObjectName("groupBoxSelectRef")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.groupBoxSelectRef)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label_2 = QtWidgets.QLabel(self.groupBoxSelectRef)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_5.addWidget(self.label_2)
        self.listAvailRef = QtWidgets.QListView(self.groupBoxSelectRef)
        self.listAvailRef.setObjectName("listAvailRef")
        self.verticalLayout_5.addWidget(self.listAvailRef)
        self.horizontalLayout_3.addLayout(self.verticalLayout_5)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.buttonSelectRef = QtWidgets.QPushButton(self.groupBoxSelectRef)
        self.buttonSelectRef.setObjectName("buttonSelectRef")
        self.verticalLayout_4.addWidget(self.buttonSelectRef)
        self.buttonUnSelectRef = QtWidgets.QPushButton(self.groupBoxSelectRef)
        self.buttonUnSelectRef.setObjectName("buttonUnSelectRef")
        self.verticalLayout_4.addWidget(self.buttonUnSelectRef)
        self.horizontalLayout_3.addLayout(self.verticalLayout_4)
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.label_3 = QtWidgets.QLabel(self.groupBoxSelectRef)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_6.addWidget(self.label_3)
        self.listSelRef = QtWidgets.QListView(self.groupBoxSelectRef)
        self.listSelRef.setObjectName("listSelRef")
        self.verticalLayout_6.addWidget(self.listSelRef)
        self.horizontalLayout_3.addLayout(self.verticalLayout_6)
        self.verticalLayout.addWidget(self.groupBoxSelectRef)
        self.groupBoxSelectModels = QtWidgets.QGroupBox(Dialog)
        self.groupBoxSelectModels.setMinimumSize(QtCore.QSize(0, 100))
        self.groupBoxSelectModels.setObjectName("groupBoxSelectModels")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.groupBoxSelectModels)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout()
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.label_4 = QtWidgets.QLabel(self.groupBoxSelectModels)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_7.addWidget(self.label_4)
        self.listCurrentModels = QtWidgets.QListView(self.groupBoxSelectModels)
        self.listCurrentModels.setObjectName("listCurrentModels")
        self.verticalLayout_7.addWidget(self.listCurrentModels)
        self.horizontalLayout_4.addLayout(self.verticalLayout_7)
        self.widget_2 = QtWidgets.QWidget(self.groupBoxSelectModels)
        self.widget_2.setMinimumSize(QtCore.QSize(180, 0))
        self.widget_2.setObjectName("widget_2")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.widget_2)
        self.verticalLayout_8.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.buttonAddSingle = QtWidgets.QPushButton(self.widget_2)
        self.buttonAddSingle.setObjectName("buttonAddSingle")
        self.verticalLayout_8.addWidget(self.buttonAddSingle)
        self.buttonGenerateComb = QtWidgets.QPushButton(self.widget_2)
        self.buttonGenerateComb.setObjectName("buttonGenerateComb")
        self.verticalLayout_8.addWidget(self.buttonGenerateComb)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_8.addItem(spacerItem1)
        self.buttonDeleteModel = QtWidgets.QPushButton(self.widget_2)
        self.buttonDeleteModel.setObjectName("buttonDeleteModel")
        self.verticalLayout_8.addWidget(self.buttonDeleteModel)
        self.buttonRemoveAll = QtWidgets.QPushButton(self.widget_2)
        self.buttonRemoveAll.setObjectName("buttonRemoveAll")
        self.verticalLayout_8.addWidget(self.buttonRemoveAll)
        self.horizontalLayout_4.addWidget(self.widget_2)
        self.verticalLayout.addWidget(self.groupBoxSelectModels)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem2)
        self.buttonCancel = QtWidgets.QPushButton(Dialog)
        self.buttonCancel.setObjectName("buttonCancel")
        self.horizontalLayout_5.addWidget(self.buttonCancel)
        self.buttonAccept = QtWidgets.QPushButton(Dialog)
        self.buttonAccept.setObjectName("buttonAccept")
        self.horizontalLayout_5.addWidget(self.buttonAccept)
        self.verticalLayout.addLayout(self.horizontalLayout_5)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtWidgets.QApplication.translate("Dialog", "Read from combined input", None, -1))
        self.groupBoxLoadFile.setTitle(QtWidgets.QApplication.translate("Dialog", "Load file", None, -1))
        self.label.setText(QtWidgets.QApplication.translate("Dialog", "Loaded file: ", None, -1))
        self.loadedFileText.setText(QtWidgets.QApplication.translate("Dialog", "None", None, -1))
        self.buttonSelectFile.setText(QtWidgets.QApplication.translate("Dialog", "Select file", None, -1))
        self.groupBoxSelectRef.setTitle(QtWidgets.QApplication.translate("Dialog", "Select reference genes", None, -1))
        self.label_2.setText(QtWidgets.QApplication.translate("Dialog", "Available reference genes", None, -1))
        self.buttonSelectRef.setText(QtWidgets.QApplication.translate("Dialog", "Select as ref->", None, -1))
        self.buttonUnSelectRef.setText(QtWidgets.QApplication.translate("Dialog", "<- Unselect", None, -1))
        self.label_3.setText(QtWidgets.QApplication.translate("Dialog", "Selected genes as reference", None, -1))
        self.groupBoxSelectModels.setTitle(QtWidgets.QApplication.translate("Dialog", "Select Models", None, -1))
        self.label_4.setText(QtWidgets.QApplication.translate("Dialog", "Current models", None, -1))
        self.buttonAddSingle.setText(QtWidgets.QApplication.translate("Dialog", "Add single", None, -1))
        self.buttonGenerateComb.setText(QtWidgets.QApplication.translate("Dialog", "Generate combinations", None, -1))
        self.buttonDeleteModel.setText(QtWidgets.QApplication.translate("Dialog", "Delete model", None, -1))
        self.buttonRemoveAll.setText(QtWidgets.QApplication.translate("Dialog", "Remove all", None, -1))
        self.buttonCancel.setText(QtWidgets.QApplication.translate("Dialog", "Cancel", None, -1))
        self.buttonAccept.setText(QtWidgets.QApplication.translate("Dialog", "Accept", None, -1))




from PySide2.QtWidgets import QMessageBox

def popup(title, text, severity="warning"):

    if severity == "warning":
        severity = QMessageBox.Warning
    elif severity == "information":
        severity = QMessageBox.Information
    elif severity == "question":
        severity = QMessageBox.Question
    else:
        severity = QMessageBox.Critical

    msg = QMessageBox()
    msg.setIcon(severity)
    msg.setText(title)
    msg.setInformativeText(text)
    msg.setWindowTitle(title)
    msg.exec_()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'TableDisplayDialog.ui',
# licensing of 'TableDisplayDialog.ui' applies.
#
# Created: Tue Jan  7 21:29:11 2020
#      by: pyside2-uic  running on PySide2 5.12.0
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(556, 449)
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.tableView = QtWidgets.QTableView(Dialog)
        self.tableView.setObjectName("tableView")
        self.gridLayout.addWidget(self.tableView, 0, 0, 1, 1)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtWidgets.QApplication.translate("Dialog", "Dialog", None, -1))




import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='GenExpA',
    version='0.0.1',
    author='Dawid Maciazek',
    description="Comprehensive tool based on the new workflow for qPCR data analysis",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    install_requires=[
        'PySide2>=5.15.2',
        'numpy>=1.19.5',
        'cycler>=0.10.0',
        'et-xmlfile>=1.1.0',
        'kiwisolver>=1.3.1',
        'matplotlib>=3.3.4',
        'openpyxl>=3.0.7',
        'pandas>=1.1.5',
        'patsy>=0.5.1',
        'Pillow>=8.2.0',
        'pkg-resources>=0.0.0',
        'pyparsing>=2.4.7',
        'python-dateutil>=2.8.1',
        'pytz>=2021.1',
        'scikit-posthocs>=0.6.7',
        'scipy>=1.5.4',
        'seaborn>=0.11.1',
        'shiboken2>=5.15.2',
        'six>=1.16.0',
        'statsmodels>=0.12.2',
        'xlrd>=2.0.1'],
    entry_points = {
        "console_scripts": [
            "genexp = genexpa.main:main"
        ]
    },
    python_requires=">=3.6",

)



from genexpa.uis.uis.TableDisplayDialogUI import Ui_Dialog
from PySide2.QtWidgets import QDialog
from PySide2 import QtCore

from genexpa.core import explib


class PandasModel(QtCore.QAbstractTableModel):
    def __init__(self, df, parent=None, str_col=[]):
        QtCore.QAbstractTableModel.__init__(self, parent=parent)
        self.str_col = str_col
        self._df = df

    def toDataFrame(self):
        return self._df.copy()

    def headerData(self, section, orientation, role=QtCore.Qt.DisplayRole):
        if role != QtCore.Qt.DisplayRole:
            return None

        if orientation == QtCore.Qt.Horizontal:
            return self._df.columns.tolist()[section]

        elif orientation == QtCore.Qt.Vertical:
            return self._df.index.tolist()[section]

    def data(self, index, role=QtCore.Qt.DisplayRole):
        if role != QtCore.Qt.DisplayRole:
            return None

        if not index.isValid():
            return None

        if index.column() in self.str_col:
            return str(self._df.iloc[index.row(), index.column()])
        return '{:0.3e}'.format(self._df.iloc[index.row(), index.column()])

    def setData(self, index, value, role):
        row = self._df.index[index.row()]
        col = self._df.columns[index.column()]
        if hasattr(value, 'toPyObject'):
            # PyQt4 gets a QVariant
            value = value.toPyObject()
        else:
            # PySide gets an unicode
            dtype = self._df[col].dtype
            if dtype != object:
                value = None if value == '' else dtype.type(value)
        self._df.set_value(row, col, value)

        return True

    def rowCount(self, parent=QtCore.QModelIndex()):
        return len(self._df.index)

    def columnCount(self, parent=QtCore.QModelIndex()):
        return len(self._df.columns)

class TableDisplayDialog(QDialog):
    def __init__(self, parent, df, name="Table", str_col=[]):
        super().__init__(parent)
        self._df = explib.flatten_mlvl_index(df)

        self.ui = Ui_Dialog()
        self.ui.setupUi(self)
        self.setWindowTitle(name)

        self.model = PandasModel(self._df, None, str_col)
        self.ui.tableView.setModel(self.model)
